/*$

=head1 NAME

Balance - Perform trace balancing
 
=head1 SYNOPSIS
 
Balance < in.H bal= oper=0 rms=2000 > out.H 

=head1 INPUT PARAMETERS

=over 4
 
=item oper -  integer 

	[0]:  compute balance factors from input data, and apply them 
		to create output.
	  1:  compute balance factors from input data, and output the 
		balance factors.
	  2:  read in balance factors from auxiliary dataset (bal=), 
		apply them to main input, and output the trace-balanced data.

=item rms  -  real    

	[2000]: desired output rms level. If rms=0, balance factors will 
		be computed to match the rms level of the first live trace.

=item n1,n2,n3-integer  

      input cube dimensions

=item bal - char*  

      name of auxiliary file containing balance factors (if oper==2)

=back

=head1 DESCRIPTION

   Trace balancing of an entire trace (no windowing).
 
=head1 EXAMPLES
 
	Balance < in.H bal=mybal.txt oper=2 > out.H 
	Apply balance factors contained in file mybal.txt

	Balance < in.H oper=0 rms=2000> out.H 
	Compute balance factors in a 2000 ms window and 
	apply to the data.

=head1 CATEGORY

B<seis/filter>

=cut
>*/

/*
 
  KEYWORDS trace-balance balance gain
 
  SEE ALSO

  WHERE
  ./cube/seis/Balance.c
 
*/
/*
Author: Steve Cole (SEP)
Revised: Steve Cole (SEP)
  Added options to output balance factors, or read in balance factors
  from aux input and apply them. Useful if you want to compute factors
  from data in one form (such as amplitude spectra over some frequency
  range) and apply them to the raw data.
Revised: D. Nichols and R. Abma (SEP)
        Corrected auxillary file input check.
Martin Karrenbach 11-3-92  changed alloc(sizeof()) and sreed/srite sizeof(xdr)
Revised: Morgan Brown (23Jan2003) - Removed switch statements (converted
	to simple if-else) which run correctly on SGI64, but not LINUX, on this
	release, and possibly 6.0.  Also added better error handling for auxiliary 
	file input.
*/
#define MY_SEP_DOC \
 sep_add_doc_line("NAME");\
 sep_add_doc_line("    Balance - Perform trace balancing");\
 sep_add_doc_line("");\
 sep_add_doc_line("SYNOPSIS");\
 sep_add_doc_line("    Balance < in.H bal= oper=0 rms=2000 > out.H");\
 sep_add_doc_line("");\
 sep_add_doc_line("INPUT PARAMETERS");\
 sep_add_doc_line("    oper - integer");\
 sep_add_doc_line("                [0]:  compute balance factors from input data, and apply them ");\
 sep_add_doc_line("                        to create output.");\
 sep_add_doc_line("                  1:  compute balance factors from input data, and output the ");\
 sep_add_doc_line("                        balance factors.");\
 sep_add_doc_line("                  2:  read in balance factors from auxiliary dataset (bal=), ");\
 sep_add_doc_line("                        apply them to main input, and output the trace-balanced data.");\
 sep_add_doc_line("");\
 sep_add_doc_line("    rms - real");\
 sep_add_doc_line("                [2000]: desired output rms level. If rms=0, balance factors will ");\
 sep_add_doc_line("                        be computed to match the rms level of the first live trace.");\
 sep_add_doc_line("");\
 sep_add_doc_line("    n1,n2,n3-integer");\
 sep_add_doc_line("              input cube dimensions");\
 sep_add_doc_line("");\
 sep_add_doc_line("    bal - char*");\
 sep_add_doc_line("              name of auxiliary file containing balance factors (if oper==2)");\
 sep_add_doc_line("");\
 sep_add_doc_line("DESCRIPTION");\
 sep_add_doc_line("       Trace balancing of an entire trace (no windowing).");\
 sep_add_doc_line("");\
 sep_add_doc_line("EXAMPLES");\
 sep_add_doc_line("            Balance < in.H bal=mybal.txt oper=2 > out.H ");\
 sep_add_doc_line("            Apply balance factors contained in file mybal.txt");\
 sep_add_doc_line("");\
 sep_add_doc_line("            Balance < in.H oper=0 rms=2000> out.H ");\
 sep_add_doc_line("            Compute balance factors in a 2000 ms window and ");\
 sep_add_doc_line("            apply to the data.");\
 sep_add_doc_line("");\
 sep_add_doc_line("CATEGORY");\
 sep_add_doc_line("    seis/filter");\
 sep_add_doc_line("");

#include <sep.main>
#include <stdio.h>
MAIN()
{
	int i1, n1, n1out;
	int i2, n2, n2out;
	int i3, n3, n3out;
	int esize ;
	float rms, mean, scale, rmsout, sum;
	int oper;
	FILE* auxf;
	float *input;
	float *output;
	float *factors;

	/* get input parameters */
	if(!hetch("n1","d",&n1)) seperr("n1");
	if(!hetch("n2","d",&n2)) seperr("n2");
	if(!hetch("n3","d",&n3)) seperr("n3");
	if(!hetch("esize","d",&esize)) esize = sizeof(float);
	rmsout = 2000.;
	getch("rms","f",&rmsout);
	oper = 0;
	getch("oper","d",&oper);

	/* allocate auxiliary arrays */ 
	input = (float *) alloc( n1 * sizeof(float));
	output = (float *) alloc( n1 * sizeof(float));
	factors = (float *) alloc( n2 * n3 * sizeof(float));

	/* set standard SEP paramters according to operation type */
	if( oper==1 ) {                          /* n1 zapped */
		n1out = n2;  n2out = n3;  n3out = 1;
	} else if( oper==2 ) {                   /* same as input */
		n1out = n1;  n2out = n2;  n3out = n3;
		if( auxin("bal") != NULL ) {           /* bal. factors from auxfile */
			if( n2*n3*esize != sreed( "bal", factors, n2*n3*esize) ) {
				seperr("FATAL: error reading from aux file bal\n");
			}
		} else {
			seperr("FATAL: must input aux file bal for oper=2\n");
		}
	} else {
		n1out = n1;  n2out = n2;  n3out = n3;  /* default */
	}

	/* output SEP standard parameters */ 
	putch("n1","d",&n1out);
	putch("n2","d",&n2out);
	putch("n3","d",&n3out);
	hclose();

	/* do the trace balancing according to the required option */ 
	/* loops over planes and over traces. Processes one trace at a time */
	for (i3=0; i3<n3; i3++) {
		for (i2=0; i2<n2; i2++) {
			sreed( "in", input, n1 * 4);	/* reads one trace */

			/* compute scaling factor */
			if( oper==2 ) {
				scale = factors[i2+i3*n2];
			} else {
				sum = 0.;
				for (i1=0; i1<n1; i1++) {
					sum += input[i1]*input[i1];
				}
				mean = sum / n1;
				rms = sqrt(mean);
				if (rms != 0.) { 
					if (rmsout == 0.) rmsout = rms;
					scale = rmsout / rms;
				} else {
					scale = 0.;
				}
			}
 
			/* apply scaling factor */
			if( oper==1 ) {
				output[0] = scale;
				srite ("out",output,esize);
			} else {
				for (i1=0; i1<n1; i1++) {
					output[i1] = input[i1]*scale;
				}
				srite ("out",output,n1*esize);	/* output one trace */
			}
		}
    }
}

