
 /*$

=head1 NAME

Attr - display attributes of a dataset

=head1 SYNOPSIS

Attr <in want=all

=head1 INPUT PARAMETERS 

=over 4

=item   want  -  char* 

       {all,min,max,mean,rms,norm}
       want=all, prints a summary of all info
       want=min, prints just the minimum value
       want=max, prints just the maximum value
       want=mean, prints just the mean value
       want=rms, prints just the rms value
       want=norm, prints just the L2 norm value
       want=short, prints ratio of 0 samples,minimum,maximum

=item   maxsize - integer 

       [50] maximum memory that will be used (in megabytes).
       Blocks of maxsize/esize elements read and analyzed
       sequentially.

=item   param - integer 

       [0] Output style. Default (param=0) is 'verbose' style 
       (see below).  param=1 formats output in SEP history
       file parameter style.  Useful for grabbing the data
       statistics for batch processing (see example below).

=back

=head1 DESCRIPTION

 Displays various attributes of a dataset
 

=head1 COMMENTS 

  sample output for want=all, param=0 is
 
    ******************************************* 
    mean value = 0.010000 
    rms = 0.100000 
    norm = 7.07106
    maximum value = 1.000000 at 10 1 1 
    minimum value = 0.000000 at 1 1 1 
    number of nonzero samples = 50 
    total number of samples = 5000 
    *******************************************

  and for want=all, param=1:

    meanval=0.010000 
    rms=0.100000 
    norm=7.07106
    maxval=1.000000
    minval=0.000000
    nonzero=50 
    totsamp=5000 

  Say that we wanted to use the maximum value in one dataset
  as a clip value for displaying another dataset.  We could
  use Attr and Get as follows.

	< data2.H  Grey clip=`Attr < data1.H | Get parform=n maxval` | Tube


=head1 SEE ALSO

L<Attr3dhead>, L<In>

=head1 CATEGORY

B<util/info>

=cut

>*/

/* 
 KEYWORDS   mean rms maximum minimum norm attribute 

 WHERE
  ./cube/util/Attr.c

*/
/*
 Modified: Dave Nichols, SEP 3/10/94
      Added alternative outputs

 Modified: Curt Holden, SEP 8/20/96
      Changed label from "l2norm" to "norm"

 Modified: Yalei Sun, SEP 9/11/96
      Took n4-n9 axes into account

 Modified: Hector Urdaneta, SEP 11/1/96
      Fixed maxv and minv to be initialized correctly

 Modified: Biondo Biondi , SEP 12/15/96
      Reverted to setting minv maxv to avoid eseking file

 Modified: Robert Clapp , SEP 6/27/97
      Added short option to make testing program easier to debug
 Modified: Robert Clapp. SEP 10/97
      Added complex and byte support, made N dimensional, added
      maxsize parameters
 Modified: Morgan Brown, SEP 9/May/2002
      Added "param" option to produce history file-like output 
      values.
 */

#define         HEAD    "/dev/null"
#define         OUT    "/dev/null"
#include<sep3d.h>
#include <math.h>

#if defined(__STDC__) || defined(__stdc__)
void uncode_location(int loc,int *block,int *loc_a,int ndim);
#else
void uncode_location();
#endif

#define MY_SEP_DOC \
 sep_add_doc_line("NAME");\
 sep_add_doc_line("    Attr - display attributes of a dataset");\
 sep_add_doc_line("");\
 sep_add_doc_line("SYNOPSIS");\
 sep_add_doc_line("    Attr <in want=all");\
 sep_add_doc_line("");\
 sep_add_doc_line("INPUT PARAMETERS");\
 sep_add_doc_line("    want - char*");\
 sep_add_doc_line("               {all,min,max,mean,rms,norm}");\
 sep_add_doc_line("               want=all, prints a summary of all info");\
 sep_add_doc_line("               want=min, prints just the minimum value");\
 sep_add_doc_line("               want=max, prints just the maximum value");\
 sep_add_doc_line("               want=mean, prints just the mean value");\
 sep_add_doc_line("               want=rms, prints just the rms value");\
 sep_add_doc_line("               want=norm, prints just the L2 norm value");\
 sep_add_doc_line("               want=short, prints ratio of 0 samples,minimum,maximum");\
 sep_add_doc_line("");\
 sep_add_doc_line("    maxsize - integer");\
 sep_add_doc_line("               [50] maximum memory that will be used (in megabytes).");\
 sep_add_doc_line("               Blocks of maxsize/esize elements read and analyzed");\
 sep_add_doc_line("               sequentially.");\
 sep_add_doc_line("");\
 sep_add_doc_line("    param - integer");\
 sep_add_doc_line("               [0] Output style. Default (param=0) is 'verbose' style ");\
 sep_add_doc_line("               (see below).  param=1 formats output in SEP history");\
 sep_add_doc_line("               file parameter style.  Useful for grabbing the data");\
 sep_add_doc_line("               statistics for batch processing (see example below).");\
 sep_add_doc_line("");\
 sep_add_doc_line("DESCRIPTION");\
 sep_add_doc_line("     Displays various attributes of a dataset");\
 sep_add_doc_line("");\
 sep_add_doc_line("COMMENTS");\
 sep_add_doc_line("      sample output for want=all, param=0 is");\
 sep_add_doc_line("");\
 sep_add_doc_line("        ******************************************* ");\
 sep_add_doc_line("        mean value = 0.010000 ");\
 sep_add_doc_line("        rms = 0.100000 ");\
 sep_add_doc_line("        norm = 7.07106");\
 sep_add_doc_line("        maximum value = 1.000000 at 10 1 1 ");\
 sep_add_doc_line("        minimum value = 0.000000 at 1 1 1 ");\
 sep_add_doc_line("        number of nonzero samples = 50 ");\
 sep_add_doc_line("        total number of samples = 5000 ");\
 sep_add_doc_line("        *******************************************");\
 sep_add_doc_line("");\
 sep_add_doc_line("      and for want=all, param=1:");\
 sep_add_doc_line("");\
 sep_add_doc_line("        meanval=0.010000 ");\
 sep_add_doc_line("        rms=0.100000 ");\
 sep_add_doc_line("        norm=7.07106");\
 sep_add_doc_line("        maxval=1.000000");\
 sep_add_doc_line("        minval=0.000000");\
 sep_add_doc_line("        nonzero=50 ");\
 sep_add_doc_line("        totsamp=5000 ");\
 sep_add_doc_line("");\
 sep_add_doc_line("      Say that we wanted to use the maximum value in one dataset");\
 sep_add_doc_line("      as a clip value for displaying another dataset.  We could");\
 sep_add_doc_line("      use Attr and Get as follows.");\
 sep_add_doc_line("");\
 sep_add_doc_line("            < data2.H  Grey clip=`Attr < data1.H | Get parform=n maxval` | Tube");\
 sep_add_doc_line("");\
 sep_add_doc_line("SEE ALSO");\
 sep_add_doc_line("    Attr3dhead, In");\
 sep_add_doc_line("");\
 sep_add_doc_line("CATEGORY");\
 sep_add_doc_line("    util/info");\
 sep_add_doc_line("");

#include <sep.main>
MAIN(){

  int esize;
  int i1,ncount;
  int nzero,ndim,maxsize,mem,param;
  float* f_databuf;
  float f_mean,f_maxv,f_minv,co_maxv,co_minv;
  complex *co_databuf,co_minv_r,co_minv_i,co_maxv_r,co_maxv_i;
  float meansngl;
  double sqrs;
  float sqrsngl;
  float norm,amplitude;
  char want[256],temp_ch[256];
  unsigned char *ch_databuf;
  int *n,*maxloc,*minloc,tot,*cmaxloc,*cminloc,count,read,read_in,*block;

  /* fetch in parameters  */
  if( getch("want",   "s", want)     == 0 ) strcpy( want, "all" );
  if( fetch("esize",  "d", &esize)   == 0 ) esize=4;
  if( getch("maxsize","d", &maxsize) == 0 ) maxsize=50;
  if( getch("param",  "d", &param)   == 0 ) param=0;

	maxsize=maxsize*1000000;


  if( getch("n1","d",&ndim) == 0 ){ 
    if( 0!=sep_get_number_data_axes("in",&ndim) ) seperr("trouble getting ndim");
  } else {
    ndim=10;  /*We have n on the command line so set an arbitrary maximum dimensions to 10*/
  }

  n       = (int *)malloc( sizeof(int)*ndim );
  block   = (int *)malloc( sizeof(int)*(ndim+1) );
  minloc  = (int *)malloc( sizeof(int)*(ndim+1));
  maxloc  = (int *)malloc( sizeof(int)*(ndim+1));
  cminloc = (int *)malloc( sizeof(int)*(ndim+1));
  cmaxloc = (int *)malloc( sizeof(int)*(ndim+1));

  tot=1;
  block[0]=1;
  for( i1=1; i1<=ndim; i1++ ) {     /* Fetch n1,n2,...ndim from history */
    sprintf( temp_ch, "%s%d", "n", i1);
    if( 0==fetch( temp_ch, "d", &n[i1-1] ))  n[i1-1]=1;
    minloc[i1]=1; maxloc[i1]=1; tot=tot*n[i1-1];
    cminloc[i1]=1; cmaxloc[i1]=1; 
    block[i1]=block[i1-1]*n[i1-1];
  }

  /*calculate the amount we can hold in at one time */
  mem=maxsize/esize;
  if( mem>tot ) mem=tot;

  /*initialize for three cases float (esize=4), 
    complex (esize=8), byte (esize=1)    */

  if(esize==4){
    f_mean = 0.; ncount = 0; nzero = 0;
    f_maxv = -SEP_MAXFLOAT; f_minv = SEP_MAXFLOAT; sqrs = 0.0;
    f_databuf = (float*)alloc((int)( mem*sizeof(float) ));
  } else if(esize==8) {
    f_mean = 0.; ncount = 0; nzero = 0; 
    f_maxv = -SEP_MAXFLOAT; f_minv = SEP_MAXFLOAT; sqrs = 0.0;
    co_maxv=-SEP_MAXFLOAT; co_minv=SEP_MAXFLOAT;
    co_databuf = (complex*)alloc((int)( mem*sizeof(complex) ));
  } else if(esize==1) {
    f_mean = 0.; ncount = 0; nzero = 0; 
    f_maxv = -SEP_MAXFLOAT; f_minv = SEP_MAXFLOAT; sqrs = 0.0;
    ch_databuf = (unsigned char*)alloc( (int)(mem*sizeof(char) ));
  } else {
    seperr("unrecognized esize, only handle esize=1,4,8\n");
  }

  hclose();

  count=0;
  while( count < tot) {                   /* read in the data */
    read=tot-count;
    if(read>mem) read=mem;
    if(     esize==4) read_in=sreed( "in", f_databuf,  esize*read);
    else if(esize==8) read_in=sreed( "in", co_databuf, esize*read);
    else if(esize==1) read_in=sreed( "in", ch_databuf, esize*read);

    if(read_in!=read*esize) {
      fprintf(stderr,
         "unexpected end of data after %d elements \n",read_in/esize+count);
      tot=count+read_in;
    }

    /*update running totals and check min max stuff */  

    if(esize==4){

      for(i1=0; i1 < read_in/esize; i1++){
        f_mean = f_mean + f_databuf[i1];
        sqrs = sqrs + f_databuf[i1]*f_databuf[i1];

        if(f_databuf[i1] > f_maxv){
          f_maxv=f_databuf[i1];
          uncode_location(i1+count,block,maxloc,ndim);
        }
        if(f_databuf[i1] < f_minv){
          f_minv=f_databuf[i1];
          uncode_location(i1+count,block,minloc,ndim);
        }
        if(f_databuf[i1] != 0) nzero=nzero+1;
      }

    } else if(esize==8){

      for(i1=0; i1 < read_in/esize; i1++){
        amplitude=sqrt(co_databuf[i1].re*co_databuf[i1].re +
         co_databuf[i1].im * co_databuf[i1].im);

        f_mean=f_mean+ amplitude;
        sqrs=sqrs+amplitude * amplitude;

        if(co_databuf[i1].re > f_maxv){
          f_maxv=co_databuf[i1].re;
          co_maxv_r=co_databuf[i1];
          uncode_location(i1+count,block,maxloc,ndim);
        }
        if(co_databuf[i1].im > co_maxv){
          co_maxv=co_databuf[i1].im;
          co_maxv_i=co_databuf[i1];
          uncode_location(i1+count,block,cmaxloc,ndim);
        }
        if(co_databuf[i1].re < f_minv){
          f_minv=co_databuf[i1].re;
          co_minv_r=co_databuf[i1];
          uncode_location(i1+count,block,minloc,ndim);
        }
        if(co_databuf[i1].im < co_minv){
          co_minv=co_databuf[i1].im;
          co_minv_i=co_databuf[i1];
          uncode_location(i1+count,block,cminloc,ndim);
        }
        if(amplitude != 0) nzero=nzero+1;
      }

    } else{
      for(i1=0; i1 < read_in/esize; i1++){
        amplitude=(float)ch_databuf[i1];
        f_mean=f_mean+ amplitude;
        sqrs=sqrs+amplitude*amplitude;

        if(amplitude > f_maxv){
          f_maxv=amplitude;
          uncode_location(i1+count,block,maxloc,ndim);
        }
        if(amplitude < f_minv){
          f_minv=amplitude;
          uncode_location(i1+count,block,minloc,ndim);
        }
        if(amplitude != 0) nzero=nzero+1;
      } 
    }

    count=count+read_in/esize;
  }

  meansngl = f_mean/count;
  sqrsngl = sqrt(sqrs/count);
  norm = sqrt(sqrs);

  if( 0==strcmp( want, "all" ) ) {
      printf("******************************************* \n");
      if(esize==8) 
      printf("   rms, mean and norm refer to amplitude    \n\n");
  }
  if(0==strcmp(want,"all") || 0==strcmp(want,"rms")) {
    if( param==0 ) printf("rms = %g \n",sqrsngl);
    else           printf("rms=%g \n",sqrsngl);
  }
  if(0==strcmp(want,"mean") || 0==strcmp(want,"all")) {
    if( param==0 ) printf("mean value = %g \n",meansngl);
    else           printf("meanval=%g \n",meansngl);
  }
  if(0==strcmp(want,"norm") || 0==strcmp(want,"all"))
    if( param==0 ) printf("norm value = %g \n",norm);
    else           printf("norm=%g \n",norm);
    if(esize==8){
    if(0==strcmp(want,"max") || 0==strcmp(want,"all")){
      if( param==0 ) printf("maximum real value = (%g,%g) at ",co_maxv_r.re,co_maxv_r.im);
      else           printf("maxval_real=(%g,%g)",co_maxv_r.re,co_maxv_r.im);
      if( param==0 ) { for(i1=0; i1<ndim; i1++) printf("%d ",maxloc[i1]); }
      printf("\n");

      if( param==0 ) printf("maximum imaginary value = (%g,%g) at ",co_maxv_i.re,co_maxv_i.im);
      else           printf("maxval_imag=(%g,%g)",co_maxv_i.re,co_maxv_i.im);
      if( param==0 ) { for(i1=0; i1<ndim; i1++) printf("%d ",cmaxloc[i1]); }
      printf("\n");
    }
    if(0==strcmp(want,"min") || 0==strcmp(want,"all")){
/*
      printf("minimum imaginary value = (%g,%g) at ",co_minv_i.re,co_minv_i.im);
      for(i1=0; i1<ndim; i1++) printf("%d ",cminloc[i1]);
      printf("\n");
      printf("minimum real value = (%g,%g) at ",co_minv_r.re,co_minv_r.im);
      for(i1=0; i1<ndim; i1++) printf("%d ",minloc[i1]);
      printf("\n"); */

      if( param==0 ) printf("minimum imaginary value = (%g,%g) at ",co_minv_i.re,co_minv_i.im);
      else           printf("minval_imag=(%g,%g)",co_minv_i.re,co_maxv_i.im);
      if( param==0 ) { for(i1=0; i1<ndim; i1++) printf("%d ",cminloc[i1]); }
      printf("\n");

      if( param==0 ) printf("minimum real value = (%g,%g) at ",co_minv_r.re,co_minv_r.im);
      else           printf("maxval_imag=(%g,%g)",co_minv_r.re,co_minv_r.im);
      if( param==0 ) { for(i1=0; i1<ndim; i1++) printf("%d ",minloc[i1]); }
      printf("\n");
    }
    }
    else{

    if(0==strcmp(want,"max") || 0==strcmp(want,"all")){
      if( param==0 ) printf("maximum value = %g at ",f_maxv);
      else           printf("maxval=%g",f_maxv);
      if( param==0 ) { for(i1=0; i1<ndim; i1++) printf("%d ",maxloc[i1]); }
      printf("\n");
    }
    if(0==strcmp(want,"min") || 0==strcmp(want,"all")){
      if( param==0 ) printf("minimum value = %g at ",f_minv);
      else           printf("minval=%g",f_minv);
      if( param==0 ) { for(i1=0; i1<ndim; i1++) printf("%d ",minloc[i1]); }
      printf("\n");}
  }

  if(0==strcmp(want,"all")){
    if( param==0 ) printf("number of nonzero samples = %d \n",nzero);
    else           printf("nonzero=%d \n",nzero);
    if( param==0 ) printf("total number of samples = %d \n",count);
    else           printf("totsamp=%d \n",count);
    printf("******************************************* \n");
  }

  if(!strcmp( want, "short")){
      printf("%6.2f  pct. zeros %g-min max-%g \n",100.*((double)(ncount-nzero))/((double)ncount),f_minv,f_maxv);  
  }

  return 0;
}

#if defined(__STDC__) || defined(__stdc__)
void  uncode_location(int loc,int *block,int *loc_a,int ndim)
#else
void  uncode_location(loc,block,loc_a,ndim)
int loc,*block,*loc_a,ndim;
#endif
{
int i1;
int current;

current=loc;
if(loc/block[ndim-1] >= block[ndim]) 
  seperr("%d - %d location outside  dataset \n");
for(i1=ndim-1; i1>=0; i1--){
  loc_a[i1]=current/block[i1];
  current=current-loc_a[i1]*block[i1];
}
for(i1=0; i1<ndim; i1++)  loc_a[i1]=loc_a[i1]+1;

}

